from .integration import (
    jwt_access_security,
    jwt_access_security_None,
    jwt_access_security_ws,
)

__all__ = ["jwt_access_security", "jwt_access_security_ws", "jwt_access_security_None"]



import logging

import aiohttp
from singleton import Singleton

from usso.core import UserData, Usso


class AsyncUssoAPI(metaclass=Singleton):
    def __init__(
        self,
        url: str = "https://api.usso.io",
        api_key: str = None,
        refresh_token: str = None,
    ):
        if url and not url.startswith("http"):
            url = f"https://{url}"
        self.url = url
        assert (
            api_key or refresh_token
        ), "Either api_key or refresh_token must be provided"
        self.api_key = api_key
        self.refresh_token = refresh_token
        self.access_token = None

    async def _refresh(self, **kwargs):
        if not self.refresh_token:
            return

        url = f"{self.url}/auth/refresh"

        if self.refresh_token:
            headers = {
                "Authorization": f"Bearer {self.refresh_token}",
                "content-type": "application/json",
            }

        async with aiohttp.ClientSession() as session:
            async with session.post(url, headers=headers) as resp:
                if kwargs.get("raise_exception", True):
                    resp.raise_for_status()
                self.access_token = (await resp.json()).get("access_token")

    def _access_valid(self) -> bool:
        if not self.access_token:
            return False

        user_data = Usso(
            jwks_url=f"{self.url}/website/jwks.json?"
        ).user_data_from_token(self.access_token)
        if user_data:
            return True
        return False

    async def _request(
        self,
        method="get",
        endpoint: str = "",
        data: dict = None,
        **kwargs,
    ) -> dict:
        url = f"{self.url}/{endpoint}"
        headers = {"content-type": "application/json"}
        if self.api_key:
            headers["x-api-key"] = self.api_key
        elif self.refresh_token:
            if not self.access_token:
                await self._refresh()
            headers["Authorization"] = f"Bearer {self.access_token}"

        async with aiohttp.ClientSession() as session:
            async with session.request(
                method,
                url,
                headers=headers,
                json=data,
            ) as resp:
                try:
                    resp_json = await resp.json()
                    resp.raise_for_status()
                except aiohttp.ClientError as e:
                    logging.error(f"Error: {e}")
                    logging.error(f"Response: {resp_json}")
                    raise e
                except Exception as e:
                    logging.error(f"Error: {e}")
                    logging.error(f"Response: {resp.text}")
                    raise e
                return await resp.json()

    async def get_users(self, **kwargs) -> list[UserData]:
        users_dict = await self._request(endpoint="website/users", **kwargs)

        return [UserData(user_id=user.get("uid"), **user) for user in users_dict]

    async def get_user(self, user_id: str, **kwargs) -> UserData:
        user_dict = await self._request(
            endpoint=f"website/users/{user_id}",
            **kwargs,
        )
        return UserData(user_id=user_dict.get("uid"), **user_dict)

    async def get_user_by_credentials(self, credentials: dict, **kwargs) -> UserData:
        user_dict = await self._request(
            endpoint="website/users/credentials",
            data=credentials,
            **kwargs,
        )
        return UserData(user_id=user_dict.get("uid"), **user_dict)

    async def create_user(self, user_data: dict, **kwargs) -> UserData:
        user_dict = await self._request(
            method="post",
            endpoint="website/users",
            data=user_data,
            **kwargs,
        )

        return UserData(user_id=user_dict.get("uid"), **user_dict)

    async def create_user_credentials(
        self, user_id: str, credentials: dict, **kwargs
    ) -> UserData:
        user_dict = await self._request(
            method="post",
            endpoint=f"website/users/{user_id}/credentials",
            data=credentials,
            **kwargs,
        )
        return UserData(user_id=user_dict.get("uid"), **user_dict)

    async def create_user_by_credentials(
        self,
        user_data: dict | None = None,
        credentials: dict | None = None,
        **kwargs,
    ) -> UserData:
        user_data = user_data or {}
        if credentials:
            user_data["authenticators"] = [credentials]
        user_dict = await self._request(
            method="post",
            endpoint="website/users",
            data=credentials,
            **kwargs,
        )
        return UserData(user_id=user_dict.get("uid"), **user_dict)

    async def get_user_payload(self, user_id: str, **kwargs) -> dict:
        return await self._request(
            endpoint=f"website/users/{user_id}/payload", **kwargs
        )

    async def update_user_payload(
        self,
        user_id: str,
        payload: dict,
        **kwargs,
    ) -> dict:
        return await self._request(
            method="patch",
            endpoint=f"website/users/{user_id}/payload",
            data=payload,
            **kwargs,
        )

    async def set_user_payload(
        self,
        user_id: str,
        payload: dict,
        **kwargs,
    ) -> dict:
        return await self._request(
            method="put",
            endpoint=f"website/users/{user_id}/payload",
            data=payload,
            **kwargs,
        )



import logging

from fastapi import Request, WebSocket
from starlette.status import HTTP_401_UNAUTHORIZED

from usso.core import UserData, Usso
from usso.exceptions import USSOException

logger = logging.getLogger("usso")


def get_request_token(request: Request | WebSocket) -> UserData | None:
    authorization = request.headers.get("Authorization")
    token = None

    if authorization:
        scheme, credentials = Usso().get_authorization_scheme_param(authorization)
        if scheme.lower() == "bearer":
            token = credentials

    else:
        cookie_token = request.cookies.get("usso_access_token")
        if cookie_token:
            token = cookie_token

    return token


def jwt_access_security_None(request: Request) -> UserData | None:
    """Return the user associated with a token value."""
    token = get_request_token(request)
    if not token:
        return None
    return Usso().user_data_from_token(token, raise_exception=False)


def jwt_access_security(request: Request) -> UserData | None:
    """Return the user associated with a token value."""
    token = get_request_token(request)
    if not token:
        raise USSOException(
            status_code=HTTP_401_UNAUTHORIZED,
            error="unauthorized",
        )

    return Usso().user_data_from_token(token)


def jwt_access_security_ws(websocket: WebSocket) -> UserData | None:
    """Return the user associated with a token value."""
    token = get_request_token(websocket)
    if not token:
        raise USSOException(
            status_code=HTTP_401_UNAUTHORIZED,
            error="unauthorized",
        )

    return Usso().user_data_from_token(token)



from datetime import datetime

import jwt
import requests


class UssoSession:
    def __init__(self, sso_refresh_url: str, refresh_token: str | None = None):
        self.sso_refresh_url = sso_refresh_url
        self.refresh_token = refresh_token
        self.session = requests.Session()
        self.access_token = None

    def _refresh(self):
        if not self.refresh_token:
            return

        response = requests.post(
            self.sso_refresh_url,
            json={"refresh_token": f"{self.refresh_token}"},
        )
        response.raise_for_status()
        return response.json()

    def get_session(self):
        if self.access_token:
            decoded_token = jwt.decode(
                self.access_token, options={"verify_signature": False}
            )
            exp = datetime.fromtimestamp(decoded_token.get("exp"))
            if exp < datetime.now():
                self.access_token = None
        if not self.access_token:
            self.access_token = self._refresh()["access_token"]
            self.session.headers.update(
                {"Authorization": f"Bearer {self.access_token}"}
            )
        return self.session



import os
import unittest

from usso.api import UssoAPI
from usso.core import UserData


class TestAPI(unittest.TestCase):
    def get_usso(self):
        return UssoAPI(
            url="https://sso.usso.io",
            api_key=os.getenv("USSO_API_KEY"),
        )

    def test_get_users(self):
        usso_api = self.get_usso()
        users = usso_api.get_users()
        self.assertIsInstance(users, list)
        for user in users:
            self.assertIsInstance(user, UserData)

    def test_get_user(self):
        usso_api = self.get_usso()
        users = usso_api.get_users()
        if len(users) == 0:
            self.skipTest("No users found")
        user = users[0]
        usso_api = self.get_usso()
        user = usso_api.get_user(user.user_id)
        self.assertIsInstance(user, UserData)

    def test_get_user_by_credentials(self):
        usso_api = self.get_usso()
        users = usso_api._request(endpoint="website/users")
        if len(users) == 0:
            self.skipTest("No users found")
        for user in users:
            for auth in user["authenticators"]:
                cred = {
                    "auth_method": auth["auth_method"],
                    "representor": auth["representor"],
                }
                user = usso_api.get_user_by_credentials(cred)
                self.assertIsInstance(user, UserData)

    def test_create_user_by_credentials(self):
        import requests

        usso_api = self.get_usso()
        telegram_id = os.getenv("TELEGRAM_ID")
        cred = {"auth_method": "telegram", "representor": telegram_id}
        try:
            usso_api.create_user_by_credentials(credentials=cred)
        except requests.HTTPError as e:
            if e.response.status_code == 400:
                if e.response.json().get("error") == "already_exists":
                    self.skipTest("Credential already exists")


if __name__ == "__main__":
    unittest.main()



# the inclusion of the tests module is not meant to offer best practices for
# testing in general, but rather to support the `find_packages` example in
# setup.py that excludes installing the "tests" package

import unittest


class TestSimple(unittest.TestCase):
    def test_import(self):
        import usso

        usso.Usso()
        usso.UserData(user_id="123")


if __name__ == "__main__":
    unittest.main()



import base64
import uuid


def b64_encode_uuid(uuid_str: uuid.UUID | str):
    uuid_UUID = uuid_str if isinstance(uuid_str, uuid.UUID) else uuid.UUID(uuid_str)
    uuid_bytes = uuid_UUID.bytes
    encoded_uuid = base64.urlsafe_b64encode(uuid_bytes).decode()
    return encoded_uuid


def b64_encode_uuid_strip(uuid_str):
    return b64_encode_uuid(uuid_str).rstrip("=")


def b64_decode_uuid(encoded_uuid):
    encoded_uuid += "=" * (4 - len(encoded_uuid) % 4)  # Add padding if needed
    decoded_uuid = base64.urlsafe_b64decode(encoded_uuid)
    uuid_obj = uuid.UUID(bytes=decoded_uuid)
    return uuid_obj



from contextlib import asynccontextmanager
from datetime import datetime

import aiohttp
import jwt


class AsyncUssoSession:
    def __init__(self, sso_refresh_url: str, refresh_token: str | None = None):
        self.sso_refresh_url = sso_refresh_url
        self.refresh_token = refresh_token
        self.access_token = None
        self.session = None  # This will hold the aiohttp session

    async def _refresh(self):
        if not self.refresh_token:
            raise ValueError("Refresh token not provided or invalid.")

        async with aiohttp.ClientSession() as session:
            async with session.post(
                self.sso_refresh_url,
                json={"refresh_token": self.refresh_token},
            ) as response:
                response.raise_for_status()
                return await response.json()

    async def _ensure_valid_token(self):
        if self.access_token:
            decoded_token = jwt.decode(
                self.access_token, options={"verify_signature": False}
            )
            exp = decoded_token.get("exp")

            if exp and datetime.fromtimestamp(exp) < datetime.now():
                self.access_token = None  # Token expired, need a new one

        if not self.access_token:
            # Get a new token if none exists or it has expired
            token_data = await self._refresh()
            self.access_token = token_data["access_token"]

            # Update headers with the new access token
            if self.session:
                self.session.headers.update(
                    {"Authorization": f"Bearer {self.access_token}"}
                )

    async def __aenter__(self):
        self.session = aiohttp.ClientSession()  # Initialize the session
        await self._ensure_valid_token()  # Ensure valid token before usage
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        if self.session:
            await self.session.close()  # Close the session properly

    @asynccontextmanager
    async def _request(self, method: str, url: str, **kwargs):
        await self._ensure_valid_token()  # Ensure valid token before any request
        async with self.session.request(method, url, **kwargs) as response:
            yield response

    def get(self, url: str, **kwargs):
        return self._request("GET", url, **kwargs)

    def post(self, url: str, **kwargs):
        return self._request("POST", url, **kwargs)

    def put(self, url: str, **kwargs):
        return self._request("PUT", url, **kwargs)

    def patch(self, url: str, **kwargs):
        return self._request("PATCH", url, **kwargs)

    def delete(self, url: str, **kwargs):
        return self._request("DELETE", url, **kwargs)

    def head(self, url: str, **kwargs):
        return self._request("HEAD", url, **kwargs)

    def options(self, url: str, **kwargs):
        return self._request("OPTIONS", url, **kwargs)

    async def close(self):
        await self.session.close()
        self.session = None



import json
import logging
import os

import cachetools.func
import jwt
from fastapi import Request, WebSocket
from pydantic import BaseModel, model_validator
from starlette.status import HTTP_401_UNAUTHORIZED

from usso.core import UserData
from usso.exceptions import USSOException

logger = logging.getLogger("usso")


class JWTConfig(BaseModel):
    jwk_url: str | None = None
    secret: str | None = None
    type: str = "RS256"
    header: dict[str, str] = {"type": "Cookie", "name": "usso_access_token"}

    def __hash__(self):
        return hash(self.model_dump_json())

    @model_validator(mode="before")
    def validate_secret(cls, data: dict):
        if not data.get("jwk_url") and not data.get("secret"):
            raise ValueError("Either jwk_url or secret must be provided")
        return data

    @classmethod
    @cachetools.func.ttl_cache(maxsize=128, ttl=10 * 60)
    def get_jwk_keys(cls, jwk_url):
        return jwt.PyJWKClient(
            jwk_url,
            headers={
                "User-Agent": "usso-python",
            },
        )

    @cachetools.func.ttl_cache(maxsize=128, ttl=10 * 60)
    def decode(self, token: str):
        if self.jwk_url:
            jwk_client = self.get_jwk_keys(self.jwk_url)
            signing_key = jwk_client.get_signing_key_from_jwt(token)
            return jwt.decode(token, signing_key.key, algorithms=[self.type])

        return jwt.decode(token, self.secret, algorithms=[self.type])


class Usso:

    def __init__(self, jwt_config: str | dict | JWTConfig | None = None):
        if jwt_config is None:
            self.jwk_url = os.getenv("USSO_JWK_URL")
            return

        if isinstance(jwt_config, str):
            jwt_config = json.loads(jwt_config)
        if isinstance(jwt_config, dict):
            jwt_config = JWTConfig(**jwt_config)

        self.jwk_url = jwt_config
        self.jwt_config = jwt_config

    def get_authorization_scheme_param(
        self,
        authorization_header_value: str | None,
    ) -> tuple[str, str]:
        if not authorization_header_value:
            return "", ""
        scheme, _, param = authorization_header_value.partition(" ")
        return scheme, param

    def user_data_from_token(self, token: str, **kwargs) -> UserData | None:
        """Return the user associated with a token value."""
        try:
            decoded = self.jwk_url.decode(token)
            if decoded["token_type"] != "access":
                raise USSOException(
                    status_code=401,
                    error="invalid_token_type",
                )
            decoded["token"] = token
            return UserData(**decoded)
        except jwt.exceptions.ExpiredSignatureError:
            if kwargs.get("raise_exception", True):
                raise USSOException(status_code=401, error="expired_signature")
        except jwt.exceptions.InvalidSignatureError:
            if kwargs.get("raise_exception", True):
                raise USSOException(status_code=401, error="invalid_signature")
        except jwt.exceptions.InvalidAlgorithmError:
            if kwargs.get("raise_exception", True):
                raise USSOException(
                    status_code=401,
                    error="invalid_algorithm",
                )
        except jwt.exceptions.InvalidIssuedAtError:
            if kwargs.get("raise_exception", True):
                raise USSOException(
                    status_code=401,
                    error="invalid_issued_at",
                )
        except jwt.exceptions.InvalidTokenError:
            if kwargs.get("raise_exception", True):
                raise USSOException(
                    status_code=401,
                    error="invalid_token",
                )
        except jwt.exceptions.InvalidKeyError:
            if kwargs.get("raise_exception", True):
                raise USSOException(
                    status_code=401,
                    error="invalid_key",
                )
        except KeyError as e:
            if kwargs.get("raise_exception", True):
                raise USSOException(
                    status_code=401,
                    error="key_error",
                    message=str(e),
                )
        except USSOException as e:
            if kwargs.get("raise_exception", True):
                raise e
        except Exception as e:
            if kwargs.get("raise_exception", True):
                raise USSOException(
                    status_code=401,
                    error="error",
                    message=str(e),
                )
            logger.error(e)

    async def jwt_access_security(self, request: Request) -> UserData | None:
        """Return the user associated with a token value."""
        kwargs = {}
        authorization = request.headers.get("Authorization")
        if authorization:
            scheme, credentials = self.get_authorization_scheme_param(authorization)
            if scheme.lower() == "bearer":
                token = credentials
                return self.user_data_from_token(token, **kwargs)

        cookie_token = request.cookies.get("usso_access_token")
        if cookie_token:
            return self.user_data_from_token(cookie_token, **kwargs)

        if kwargs.get("raise_exception", True):
            raise USSOException(
                status_code=HTTP_401_UNAUTHORIZED,
                error="unauthorized",
            )
        return None

    async def jwt_access_security_ws(self, websocket: WebSocket) -> UserData | None:
        """Return the user associated with a token value."""
        kwargs = {}
        authorization = websocket.headers.get("Authorization")
        if authorization:
            scheme, credentials = self.get_authorization_scheme_param(authorization)
            if scheme.lower() == "bearer":
                token = credentials
                return self.user_data_from_token(token, **kwargs)

        cookie_token = websocket.cookies.get("usso_access_token")
        if cookie_token:
            return self.user_data_from_token(cookie_token, **kwargs)

        if kwargs.get("raise_exception", True):
            raise USSOException(
                status_code=HTTP_401_UNAUTHORIZED,
                error="unauthorized",
            )
        return None



import logging

from django.conf import settings
from django.contrib.auth.models import User
from django.db.utils import IntegrityError
from django.http import JsonResponse
from django.http.request import HttpRequest
from django.utils.deprecation import MiddlewareMixin

from usso import UserData, Usso, USSOException

logger = logging.getLogger("usso")


class USSOAuthenticationMiddleware(MiddlewareMixin):

    def process_request(self, request: HttpRequest):
        """
        Middleware to authenticate users by JWT token and create or return a user in the database.
        """
        try:
            if hasattr(request, "user") and request.user.is_authenticated:
                return

            user_data = self.jwt_access_security_none(request)
            if user_data:
                user = self.get_or_create_user(user_data)
                request.user = user
                request._dont_enforce_csrf_checks = True
        except USSOException as e:
            # Handle any errors raised by USSO authentication
            return JsonResponse({"error": str(e)}, status=401)

    def get_request_token(self, request: HttpRequest) -> str | None:
        authorization = request.headers.get("Authorization")
        if authorization:
            scheme, credentials = Usso(
                jwks_url=settings.USSO_JWK_URL
            ).get_authorization_scheme_param(authorization)
            if scheme.lower() == "bearer":
                return credentials  # Bearer token

        return request.COOKIES.get("usso_access_token")

    def jwt_access_security_none(self, request: HttpRequest) -> UserData | None:
        """Return the user associated with a token value."""
        token = self.get_request_token(request)
        if not token:
            return None
        return Usso(jwks_url=settings.USSO_JWK_URL).user_data_from_token(
            token, raise_exception=False
        )

    def jwt_access_security(self, request: HttpRequest) -> UserData | None:
        """Return the user associated with a token value."""
        token = self.get_request_token(request)
        if not token:
            raise USSOException(
                status_code=401,
                error="unauthorized",
            )

        # Get user data from the token
        return Usso(jwks_url=settings.USSO_JWK_URL).user_data_from_token(token)

    def get_or_create_user(self, user_data: UserData) -> User:
        """
        Check if a user exists by phone. If not, create a new user and return it.
        """
        phone = user_data.phone
        email = user_data.email or f"{user_data.user_id}@example.com"  # Fallback email

        try:
            # Try to get the user by phone
            user, created = User.objects.get_or_create(
                username=phone,
                defaults={
                    "first_name": user_data.user_id,
                    "email": email,
                },
            )

            if created:
                logger.info(f"New user created with phone: {phone}")

            return user

        except IntegrityError as e:
            logger.error(f"Integrity error while creating user: {str(e)}")
            raise ValueError(f"Error while creating user: {str(e)}")



import unittest
import uuid

from usso.core import Usso
from usso.exceptions import USSOException


def generate_expired_token():
    # Your code to generate an expired token goes here
    pass


# Generate an invalid token for testing
def generate_invalid_token():
    # Your code to generate an invalid token goes here
    pass


# Generate a valid token for testing
def generate_valid_token():
    # Your code to generate a valid token goes here
    pass


class TestCore(unittest.TestCase):
    def test_user_data_from_token_valid_token(self):
        return
        # Generate a valid token for testing
        valid_token = generate_valid_token()

        # Call the user_data_from_token function with the valid token
        user_data = Usso().user_data_from_token(valid_token)

        # Assert that the user_data is not None
        self.assertIsNotNone(user_data)

        # Assert that the user_data has the expected attributes
        self.assertEqual(user_data.uid, uuid.UUID(""))
        self.assertEqual(user_data.token, valid_token)
        # Add more assertions for other attributes
        # Generate an expired token for testing

    def test_user_data_from_token_expired_token(self):
        return

        # Generate an expired token for testing
        expired_token = generate_expired_token()

        # Call the user_data_from_token function with the expired token
        user_data = Usso().user_data_from_token(expired_token)

        # Assert that the user_data is None
        self.assertIsNone(user_data)

        # Assert that the USSOException is raised with the expected error
        with self.assertRaises(USSOException) as context:
            Usso().user_data_from_token(expired_token, raise_exception=True)
        self.assertEqual(context.exception.error, "expired_signature")

    def test_user_data_from_token_invalid_token(self):
        return

        # Generate an invalid token for testing
        invalid_token = generate_invalid_token()

        # Call the user_data_from_token function with the invalid token
        user_data = Usso().user_data_from_token(invalid_token)

        # Assert that the user_data is None
        self.assertIsNone(user_data)

        # Assert that the USSOException is raised with the expected error
        with self.assertRaises(USSOException) as context:
            Usso().user_data_from_token(invalid_token, raise_exception=True)
        self.assertEqual(context.exception.error, "invalid_signature")

    # Add more test cases for other scenarios


if __name__ == "__main__":
    unittest.main()



import logging
import os
import uuid
from functools import lru_cache
from typing import Optional, Tuple

import jwt
from pydantic import BaseModel
from singleton import Singleton

from . import b64tools
from .exceptions import USSOException

logger = logging.getLogger("usso")


class UserData(BaseModel):
    user_id: str
    email: str | None = None
    phone: str | None = None
    authentication_method: str | None = None
    is_active: bool = False
    jti: str | None = None
    data: dict | None = None
    token: str | None = None

    @property
    def uid(self) -> uuid.UUID:
        user_id = self.user_id

        if user_id.startswith("u_"):
            user_id = user_id[2:]
        if 22 <= len(user_id) <= 24:
            user_id = b64tools.b64_decode_uuid(user_id)

        return uuid.UUID(user_id)

    @property
    def b64id(self) -> uuid.UUID:
        return b64tools.b64_encode_uuid_strip(self.uid)


class Usso(metaclass=Singleton):
    def __init__(self, jwks_url: str | None = None):
        if jwks_url is None:
            jwks_url = os.getenv("USSO_JWKS_URL")
        self.jwks_url = jwks_url

    @lru_cache
    def get_jwks_keys(self):
        return jwt.PyJWKClient(
            self.jwks_url,
            headers={
                "User-Agent": "usso-python",
            },
        )

    def get_authorization_scheme_param(
        self,
        authorization_header_value: Optional[str],
    ) -> Tuple[str, str]:
        if not authorization_header_value:
            return "", ""
        scheme, _, param = authorization_header_value.partition(" ")
        return scheme, param

    def user_data_from_token(self, token: str, **kwargs) -> UserData | None:
        """Return the user associated with a token value."""
        try:
            # header = jwt.get_unverified_header(token)
            # jwks_url = header["jwk_url"]
            jwks_client = self.get_jwks_keys()
            signing_key = jwks_client.get_signing_key_from_jwt(token)
            decoded = jwt.decode(
                token,
                signing_key.key,
                algorithms=["RS256"],
            )
            if decoded["token_type"] != "access":
                raise USSOException(
                    status_code=401,
                    error="invalid_token_type",
                )
            decoded["token"] = token
            return UserData(**decoded)
        except jwt.exceptions.ExpiredSignatureError:
            if kwargs.get("raise_exception", True):
                raise USSOException(status_code=401, error="expired_signature")
        except jwt.exceptions.InvalidSignatureError:
            if kwargs.get("raise_exception", True):
                raise USSOException(status_code=401, error="invalid_signature")
        except jwt.exceptions.InvalidAlgorithmError:
            if kwargs.get("raise_exception", True):
                raise USSOException(
                    status_code=401,
                    error="invalid_algorithm",
                )
        except jwt.exceptions.InvalidIssuedAtError:
            if kwargs.get("raise_exception", True):
                raise USSOException(
                    status_code=401,
                    error="invalid_issued_at",
                )
        except jwt.exceptions.InvalidTokenError:
            if kwargs.get("raise_exception", True):
                raise USSOException(
                    status_code=401,
                    error="invalid_token",
                )
        except jwt.exceptions.InvalidKeyError:
            if kwargs.get("raise_exception", True):
                raise USSOException(
                    status_code=401,
                    error="invalid_key",
                )
        except USSOException as e:
            if kwargs.get("raise_exception", True):
                raise e
        except Exception as e:
            if kwargs.get("raise_exception", True):
                raise USSOException(
                    status_code=401,
                    error="error",
                    message=str(e),
                )
            logger.error(e)



error_messages = {
    "invalid_signature": "Invalid signature",
    "invalid_token": "Invalid token",
    "expired_signature": "Expired signature",
    "unauthorized": "Unauthorized",
}


class USSOException(Exception):
    def __init__(self, status_code: int, error: str, message: str = None):
        self.status_code = status_code
        self.error = error
        self.message = message
        if message is None:
            self.message = error_messages[error]
        super().__init__(message)
